@@ -22,6 +22,7 @@
 #include "config.h"
 #endif
 
+#include "gstglelements.h"
 #include "gstglsrcbin.h"
 
 GST_DEBUG_CATEGORY (gst_debug_gl_src_bin);
@@ -62,6 +63,8 @@ G_DEFINE_TYPE_WITH_CODE (GstGLSrcBin, gst_gl_src_bin,
     GST_TYPE_BIN,
     GST_DEBUG_CATEGORY_INIT (gst_debug_gl_src_bin, "glsrcbin", 0,
         "OpenGL Video Src Bin"));
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (glsrcbin, "glsrcbin",
+    GST_RANK_NONE, GST_TYPE_GL_SRC_BIN, gl_element_init (plugin));
 
 static void
 gst_gl_src_bin_class_init (GstGLSrcBinClass * klass)
@@ -98,7 +101,7 @@ gst_gl_src_bin_class_init (GstGLSrcBinClass * klass)
       g_signal_new ("create-element", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, GST_TYPE_ELEMENT, 0);
 
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "GL Src Bin", "Src/Video",
       "Infrastructure to process GL textures",
       "Matthew Waters <matthew@centricular.com>");
@@ -253,8 +256,8 @@ gst_gl_src_bin_change_state (GstElement * element, GstStateChange transition)
   GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
   GST_DEBUG ("changing state: %s => %s",
-      gst_element_state_get_name (GST_STATE_TRANSITION_CURRENT (transition)),
-      gst_element_state_get_name (GST_STATE_TRANSITION_NEXT (transition)));
+      gst_state_get_name (GST_STATE_TRANSITION_CURRENT (transition)),
+      gst_state_get_name (GST_STATE_TRANSITION_NEXT (transition)));
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
