@@ -176,6 +176,8 @@ GST_STATIC_PAD_TEMPLATE ("src",
 
 #define gst_theora_enc_parent_class parent_class
 G_DEFINE_TYPE (GstTheoraEnc, gst_theora_enc, GST_TYPE_VIDEO_ENCODER);
+GST_ELEMENT_REGISTER_DEFINE (theoraenc, "theoraenc",
+    GST_RANK_PRIMARY, GST_TYPE_THEORA_ENC);
 
 static gboolean theora_enc_start (GstVideoEncoder * enc);
 static gboolean theora_enc_stop (GstVideoEncoder * enc);
@@ -497,9 +499,9 @@ theora_enc_get_supported_formats (void)
     const char *fourcc;
   } formats[] = {
     {
-    TH_PF_420, "I420"}, {
-    TH_PF_422, "Y42B"}, {
-    TH_PF_444, "Y444"}
+        TH_PF_420, "I420"}, {
+        TH_PF_422, "Y42B"}, {
+        TH_PF_444, "Y444"}
   };
   GString *string = NULL;
   guint i;
@@ -969,7 +971,7 @@ theora_enc_handle_frame (GstVideoEncoder * benc, GstVideoCodecFrame * frame)
 
   {
     th_ycbcr_buffer ycbcr;
-    gint res, keyframe_interval;
+    gint keyframe_interval, res GST_UNUSED_ASSERT;
     GstVideoFrame vframe;
 
     if (force_keyframe) {
@@ -1049,7 +1051,7 @@ encoder_disabled:
   }
 }
 
-static gboolean
+static GstFlowReturn
 theora_enc_finish (GstVideoEncoder * benc)
 {
   GstTheoraEnc *enc;
@@ -1069,7 +1071,7 @@ theora_enc_finish (GstVideoEncoder * benc)
 
   theora_enc_clear_multipass_cache (enc);
 
-  return TRUE;
+  return GST_FLOW_OK;
 }
 
 static gboolean
@@ -1135,6 +1137,7 @@ theora_enc_set_property (GObject * object, guint prop_id,
       enc->rate_buffer = g_value_get_int (value);
       break;
     case PROP_MULTIPASS_CACHE_FILE:
+      g_free (enc->multipass_cache_file);
       enc->multipass_cache_file = g_value_dup_string (value);
       break;
     case PROP_MULTIPASS_MODE:
