@@ -25,10 +25,6 @@
 #include <stdio.h>
 #include <math.h>
 
-#ifdef HAVE_ORC
-#include <orc/orc.h>
-#endif
-
 #include "audio-resampler.h"
 #include "audio-resampler-private.h"
 #include "audio-resampler-macros.h"
@@ -257,7 +253,7 @@ convert_taps_##type##_c (gdouble *tmp_taps, gpointer taps,              \
   for (j = 0; j < n_taps; j++)                                          \
     t[j] = floor (offset + tmp_taps[j] * multiplier / weight);          \
   if (!exact)                                                           \
-    GST_WARNING ("can't find exact taps");                              \
+    GST_DEBUG ("can't find exact taps");                                \
 }
 
 #define MAKE_CONVERT_TAPS_FLOAT_FUNC(type)                              \
@@ -822,58 +818,29 @@ static ResampleFunc resample_funcs[] = {
 #define resample_gfloat_cubic_1 resample_funcs[14]
 #define resample_gdouble_cubic_1 resample_funcs[15]
 
-#if defined HAVE_ORC && !defined DISABLE_ORC
-# if defined (HAVE_ARM_NEON)
+#if defined (HAVE_ARM_NEON)
 #  define CHECK_NEON
 #  include "audio-resampler-neon.h"
-# endif
-# if defined (__i386__) || defined (__x86_64__)
+#endif
+#if defined (HAVE_SSE) || defined(HAVE_SSE2) || defined(HAVE_SSE41)
 #  define CHECK_X86
 #  include "audio-resampler-x86.h"
-# endif
 #endif
 
 static void
 audio_resampler_init (void)
 {
   static gsize init_gonce = 0;
-
   if (g_once_init_enter (&init_gonce)) {
 
     GST_DEBUG_CATEGORY_INIT (audio_resampler_debug, "audio-resampler", 0,
         "audio-resampler object");
 
-#if defined HAVE_ORC && !defined DISABLE_ORC
-    orc_init ();
-    {
-      OrcTarget *target = orc_target_get_default ();
-      gint i;
-
-      if (target) {
-        const gchar *name;
-        unsigned int flags = orc_target_get_default_flags (target);
-
-        for (i = -1; i < 32; ++i) {
-          if (i == -1) {
-            name = orc_target_get_name (target);
-            GST_DEBUG ("target %s, default flags %08x", name, flags);
-          } else if (flags & (1U << i)) {
-            name = orc_target_get_flag_name (target, i);
-            GST_DEBUG ("target flag %s", name);
-          } else
-            name = NULL;
-
-          if (name) {
 #ifdef CHECK_X86
-            audio_resampler_check_x86 (name);
+    audio_resampler_check_x86 ();
 #endif
 #ifdef CHECK_NEON
-            audio_resampler_check_neon (name);
-#endif
-          }
-        }
-      }
-    }
+    audio_resampler_check_neon ();
 #endif
     g_once_init_leave (&init_gonce, 1);
   }
@@ -1342,8 +1309,7 @@ gst_audio_resampler_options_set_quality (GstAudioResamplerMethod method,
  *
  * Make a new resampler.
  *
- * Returns: (skip) (transfer full): The new #GstAudioResampler, or
- * %NULL on failure.
+ * Returns: (skip) (transfer full): The new #GstAudioResampler.
  */
 GstAudioResampler *
 gst_audio_resampler_new (GstAudioResamplerMethod method,
@@ -1367,7 +1333,7 @@ gst_audio_resampler_new (GstAudioResamplerMethod method,
 
   audio_resampler_init ();
 
-  resampler = g_slice_new0 (GstAudioResampler);
+  resampler = g_new0 (GstAudioResampler, 1);
   resampler->method = method;
   resampler->flags = flags;
   resampler->format = format;
@@ -1622,8 +1588,6 @@ gst_audio_resampler_update (GstAudioResampler * resampler,
  * @resampler: a #GstAudioResampler
  *
  * Free a previously allocated #GstAudioResampler @resampler.
- *
- * Since: 1.6
  */
 void
 gst_audio_resampler_free (GstAudioResampler * resampler)
@@ -1637,7 +1601,7 @@ gst_audio_resampler_free (GstAudioResampler * resampler)
   g_free (resampler->sbuf);
   if (resampler->options)
     gst_structure_free (resampler->options);
-  g_slice_free (GstAudioResampler, resampler);
+  g_free (resampler);
 }
 
 /**
@@ -1648,7 +1612,7 @@ gst_audio_resampler_free (GstAudioResampler * resampler)
  * Get the number of output frames that would be currently available when
  * @in_frames are given to @resampler.
  *
- * Returns: The number of frames that would be availabe after giving
+ * Returns: The number of frames that would be available after giving
  * @in_frames as input to @resampler.
  */
 gsize
@@ -1664,12 +1628,16 @@ gst_audio_resampler_get_out_frames (GstAudioResampler * resampler,
   GST_LOG ("need %d = %d + %d + %d, avail %d = %d + %d", (gint) need,
       resampler->n_taps, resampler->samp_index, resampler->skip,
       (gint) avail, (gint) resampler->samples_avail, (gint) in_frames);
-  if (avail < need)
+  if (avail < need) {
+    GST_LOG ("avail %d < need %d", (int) avail, (int) need);
     return 0;
+  }
 
   out = (avail - need) * resampler->out_rate;
-  if (out < resampler->samp_phase)
+  if (out < resampler->samp_phase) {
+    GST_LOG ("out %d < samp_phase %d", (int) out, (int) resampler->samp_phase);
     return 0;
+  }
 
   out = ((out - resampler->samp_phase) / resampler->in_rate) + 1;
   GST_LOG ("out %d = ((%d * %d - %d) / %d) + 1", (gint) out,
@@ -1777,7 +1745,10 @@ gst_audio_resampler_resample (GstAudioResampler * resampler,
   resampler->samples_avail = samples_avail += in_frames;
 
   need = resampler->n_taps + resampler->samp_index;
-  if (G_UNLIKELY (samples_avail < need)) {
+  if (G_UNLIKELY (samples_avail < need || out_frames == 0)) {
+    GST_LOG ("not enough samples to start: need %" G_GSIZE_FORMAT ", avail %"
+        G_GSIZE_FORMAT ", out %" G_GSIZE_FORMAT, need, samples_avail,
+        out_frames);
     /* not enough samples to start */
     return;
   }
