@@ -59,6 +59,10 @@ GST_DEBUG_CATEGORY_STATIC (GST_CAT_GL_RENDERBUFFER);
 G_DEFINE_TYPE (GstGLRenderbufferAllocator, gst_gl_renderbuffer_allocator,
     GST_TYPE_GL_BASE_MEMORY_ALLOCATOR);
 
+#ifndef GST_REMOVE_DEPRECATED
+GST_DEFINE_MINI_OBJECT_TYPE (GstGLRenderbuffer, gst_gl_renderbuffer);
+#endif
+
 static guint
 _new_renderbuffer (GstGLContext * context, guint format, guint width,
     guint height)
@@ -113,7 +117,7 @@ _gl_rbo_create (GstGLRenderbuffer * gl_mem, GError ** error)
 static void
 gst_gl_renderbuffer_init (GstGLRenderbuffer * mem, GstAllocator * allocator,
     GstMemory * parent, GstGLContext * context,
-    GstGLFormat renderbuffer_format, GstAllocationParams * params,
+    GstGLFormat renderbuffer_format, const GstAllocationParams * params,
     guint width, guint height, gpointer user_data, GDestroyNotify notify)
 {
   gsize size;
@@ -377,7 +381,7 @@ static gboolean
     (GstGLRenderbufferAllocationParams * params, gsize struct_size,
     guint alloc_flags, GstGLAllocationParamsCopyFunc copy,
     GstGLAllocationParamsFreeFunc free, GstGLContext * context,
-    GstAllocationParams * alloc_params, guint width, guint height,
+    const GstAllocationParams * alloc_params, guint width, guint height,
     GstGLFormat renderbuffer_format, gpointer wrapped_data,
     gpointer gl_handle, gpointer user_data, GDestroyNotify notify)
 {
@@ -414,7 +418,7 @@ static gboolean
  */
 GstGLRenderbufferAllocationParams *
 gst_gl_renderbuffer_allocation_params_new (GstGLContext * context,
-    GstAllocationParams * alloc_params, GstGLFormat renderbuffer_format,
+    const GstAllocationParams * alloc_params, GstGLFormat renderbuffer_format,
     guint width, guint height)
 {
   GstGLRenderbufferAllocationParams *params =
@@ -453,7 +457,7 @@ gst_gl_renderbuffer_allocation_params_new (GstGLContext * context,
  */
 GstGLRenderbufferAllocationParams *
 gst_gl_renderbuffer_allocation_params_new_wrapped (GstGLContext * context,
-    GstAllocationParams * alloc_params, GstGLFormat renderbuffer_format,
+    const GstAllocationParams * alloc_params, GstGLFormat renderbuffer_format,
     guint width, guint height, gpointer gl_handle, gpointer user_data,
     GDestroyNotify notify)
 {
