@@ -74,12 +74,24 @@ G_DEFINE_TYPE_WITH_CODE (GstGLSLStage, gst_glsl_stage, GST_TYPE_OBJECT,
     GST_DEBUG_CATEGORY_INIT (gst_glsl_stage_debug, "glslstage", 0,
         "GLSL Stage"););
 
+static void
+_delete_shader (GstGLContext * context, GstGLSLStage * stage)
+{
+  GstGLSLStagePrivate *priv = stage->priv;
+
+  if (priv->handle)
+    priv->vtable.DeleteShader (priv->handle);
+}
+
 static void
 gst_glsl_stage_finalize (GObject * object)
 {
   GstGLSLStage *stage = GST_GLSL_STAGE (object);
   gint i;
 
+  gst_gl_context_thread_add (stage->context,
+      (GstGLContextThreadFunc) _delete_shader, stage);
+
   if (stage->context) {
     gst_object_unref (stage->context);
     stage->context = NULL;
@@ -133,6 +145,7 @@ gst_glsl_stage_init (GstGLSLStage * stage)
   stage->priv = gst_glsl_stage_get_instance_private (stage);
 }
 
+#ifndef G_DISABLE_CHECKS
 static gboolean
 _is_valid_shader_type (GLenum type)
 {
@@ -148,6 +161,7 @@ _is_valid_shader_type (GLenum type)
       return FALSE;
   }
 }
+#endif
 
 static const gchar *
 _shader_type_to_string (GLenum type)
@@ -261,7 +275,7 @@ gst_glsl_stage_new (GstGLContext * context, guint type)
 }
 
 /**
- * gst_glsl_stage_new_with_default_vertex:
+ * gst_glsl_stage_new_default_vertex:
  * @context: a #GstGLContext
  *
  * Returns: (transfer floating): a new #GstGLSLStage with the default vertex shader
@@ -278,7 +292,7 @@ gst_glsl_stage_new_default_vertex (GstGLContext * context)
 }
 
 /**
- * gst_glsl_stage_new_with_default_fragment:
+ * gst_glsl_stage_new_default_fragment:
  * @context: a #GstGLContext
  *
  * Returns: (transfer floating): a new #GstGLSLStage with the default fragment shader
