@@ -25,7 +25,7 @@
 #include "gstplaysinkconvertbin.h"
 
 #include <gst/pbutils/pbutils.h>
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 
 GST_DEBUG_CATEGORY_STATIC (gst_play_sink_convert_bin_debug);
 #define GST_CAT_DEFAULT gst_play_sink_convert_bin_debug
@@ -117,7 +117,6 @@ gst_play_sink_convert_bin_add_identity (GstPlaySinkConvertBin * self)
         (_("Missing element '%s' - check your GStreamer installation."),
             "identity"), (self->audio ?
             "audio rendering might fail" : "video rendering might fail")
-
         );
   } else {
     g_object_set (self->identity, "silent", TRUE, "signal-handoffs", FALSE,
@@ -350,6 +349,42 @@ gst_play_sink_convert_bin_sink_setcaps (GstPlaySinkConvertBin * self,
   }                                                                           \
 } G_STMT_END
 
+static gboolean
+gst_play_sink_convert_bin_acceptcaps (GstPad * pad, GstCaps * caps)
+{
+  GstPlaySinkConvertBin *self =
+      GST_PLAY_SINK_CONVERT_BIN (gst_pad_get_parent (pad));
+  gboolean ret;
+  GstPad *otherpad;
+
+  GST_PLAY_SINK_CONVERT_BIN_LOCK (self);
+  if (pad == self->srcpad) {
+    otherpad = self->sinkpad;
+  } else if (pad == self->sinkpad) {
+    otherpad = self->srcpad;
+  } else {
+    GST_ERROR_OBJECT (pad, "Not one of our pads");
+    otherpad = NULL;
+  }
+
+  if (otherpad) {
+    ret = gst_pad_peer_query_accept_caps (otherpad, caps);
+    if (!ret && self->converter_caps) {
+      /* maybe we can convert */
+      ret = gst_caps_can_intersect (caps, self->converter_caps);
+    }
+  } else {
+    ret = TRUE;
+  }
+  GST_PLAY_SINK_CONVERT_BIN_UNLOCK (self);
+
+  gst_object_unref (self);
+
+  GST_DEBUG_OBJECT (pad, "Accept caps: '%" GST_PTR_FORMAT "' %d", caps, ret);
+
+  return ret;
+}
+
 static GstCaps *
 gst_play_sink_convert_bin_getcaps (GstPad * pad, GstCaps * filter)
 {
@@ -474,6 +509,17 @@ gst_play_sink_convert_bin_query (GstPad * pad, GstObject * parent,
       res = TRUE;
       break;
     }
+    case GST_QUERY_ACCEPT_CAPS:
+    {
+      gboolean ret;
+      GstCaps *caps;
+
+      gst_query_parse_accept_caps (query, &caps);
+      ret = gst_play_sink_convert_bin_acceptcaps (pad, caps);
+      gst_query_set_accept_caps_result (query, ret);
+      res = TRUE;
+      break;
+    }
     default:
       res = gst_pad_query_default (pad, parent, query);
       break;
@@ -608,10 +654,8 @@ gst_play_sink_convert_bin_class_init (GstPlaySinkConvertBinClass * klass)
   gobject_class->dispose = gst_play_sink_convert_bin_dispose;
   gobject_class->finalize = gst_play_sink_convert_bin_finalize;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&srctemplate));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sinktemplate));
+  gst_element_class_add_static_pad_template (gstelement_class, &srctemplate);
+  gst_element_class_add_static_pad_template (gstelement_class, &sinktemplate);
   gst_element_class_set_static_metadata (gstelement_class,
       "Player Sink Converter Bin", "Bin/Converter",
       "Convenience bin for audio/video conversion",
