@@ -529,8 +529,8 @@ gst_uri_decode_bin3_class_init (GstURIDecodeBin3Class * klass)
           DEFAULT_INSTANT_URI, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstURIDecodebin3::select-stream
-   * @decodebin: a #GstURIDecodebin3
+   * GstURIDecodeBin3::select-stream:
+   * @decodebin: a #GstURIDecodeBin3
    * @collection: a #GstStreamCollection
    * @stream: a #GstStream
    *
@@ -1626,40 +1626,48 @@ gst_uri_decode_bin3_get_property (GObject * object, guint prop_id,
   switch (prop_id) {
     case PROP_URI:
     {
+      GST_OBJECT_LOCK (dec);
       GstPlayItem *item = dec->play_items->data;
       /* Return from the head */
       if (item->main_item)
         g_value_set_string (value, item->main_item->uri);
       else
         g_value_set_string (value, NULL);
+      GST_OBJECT_UNLOCK (dec);
       break;
     }
     case PROP_CURRENT_URI:
     {
+      GST_OBJECT_LOCK (dec);
       if (dec->output_item && dec->output_item->main_item) {
         g_value_set_string (value, dec->output_item->main_item->uri);
       } else {
         g_value_set_string (value, NULL);
       }
+      GST_OBJECT_UNLOCK (dec);
       break;
     }
     case PROP_SUBURI:
     {
+      GST_OBJECT_LOCK (dec);
       GstPlayItem *item = dec->play_items->data;
       /* Return from the head */
       if (item->sub_item)
         g_value_set_string (value, item->sub_item->uri);
       else
         g_value_set_string (value, NULL);
+      GST_OBJECT_UNLOCK (dec);
       break;
     }
     case PROP_CURRENT_SUBURI:
     {
+      GST_OBJECT_LOCK (dec);
       if (dec->output_item && dec->output_item->sub_item) {
         g_value_set_string (value, dec->output_item->sub_item->uri);
       } else {
         g_value_set_string (value, NULL);
       }
+      GST_OBJECT_UNLOCK (dec);
       break;
     }
     case PROP_CONNECTION_SPEED:
