@@ -273,6 +273,11 @@ gst_sub_parse_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
         goto beach;
       }
 
+      /* Forward seek event first and return if succeeded */
+      ret = gst_pad_event_default (pad, parent, g_steal_pointer (&event));
+      if (ret)
+        break;
+
       /* Convert that seek to a seeking in bytes at position 0,
          FIXME: could use an index */
       ret = gst_pad_push_event (self->sinkpad,
@@ -294,7 +299,6 @@ gst_sub_parse_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
         GST_WARNING_OBJECT (self, "seek to 0 bytes failed");
       }
 
-      gst_event_unref (event);
       break;
     }
     default:
