@@ -131,6 +131,8 @@ G_DEFINE_TYPE (GstVideoTestSrc, gst_video_test_src, GST_TYPE_PUSH_SRC);
 GST_ELEMENT_REGISTER_DEFINE (videotestsrc, "videotestsrc",
     GST_RANK_NONE, GST_TYPE_VIDEO_TEST_SRC);
 
+static void gst_video_test_src_finalize (GObject * object);
+
 static void gst_video_test_src_set_pattern (GstVideoTestSrc * videotestsrc,
     int pattern_type);
 static void gst_video_test_src_set_property (GObject * object, guint prop_id,
@@ -265,6 +267,7 @@ gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
 
   gobject_class->set_property = gst_video_test_src_set_property;
   gobject_class->get_property = gst_video_test_src_get_property;
+  gobject_class->finalize = gst_video_test_src_finalize;
 
   g_object_class_install_property (gobject_class, PROP_PATTERN,
       g_param_spec_enum ("pattern", "Pattern",
@@ -402,6 +405,9 @@ gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
   gst_type_mark_as_plugin_api (GST_TYPE_VIDEO_TEST_SRC_ANIMATION_MODE, 0);
   gst_type_mark_as_plugin_api (GST_TYPE_VIDEO_TEST_SRC_MOTION_TYPE, 0);
   gst_type_mark_as_plugin_api (GST_TYPE_VIDEO_TEST_SRC_PATTERN, 0);
+
+  GST_DEBUG_CATEGORY_INIT (video_test_src_debug, "videotestsrc", 0,
+      "Video Test Source");
 }
 
 static void
@@ -423,6 +429,15 @@ gst_video_test_src_init (GstVideoTestSrc * src)
   src->motion_type = DEFAULT_MOTION_TYPE;
   src->flip = DEFAULT_FLIP;
 
+  g_mutex_init (&src->cache_lock);
+}
+
+static void
+gst_video_test_src_finalize (GObject * object)
+{
+  GstVideoTestSrc *src = GST_VIDEO_TEST_SRC (object);
+  g_mutex_clear (&src->cache_lock);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static GstCaps *
@@ -762,9 +777,11 @@ gst_video_test_src_set_property (GObject * object, guint prop_id,
   }
 
   if (invalidate) {
+    g_mutex_lock (&src->cache_lock);
     /* Property change invalidated the current pattern - check if it's static now or not */
     src->have_static_pattern = gst_video_test_src_is_static_pattern (src);
     gst_clear_buffer (&src->cached);
+    g_mutex_unlock (&src->cache_lock);
   }
 }
 
@@ -969,6 +986,11 @@ gst_video_test_src_decide_allocation (GstBaseSrc * bsrc, GstQuery * query)
       pool = gst_buffer_pool_new ();
     else
       pool = gst_video_buffer_pool_new ();
+    {
+      gchar *name = g_strdup_printf ("%s-pool", GST_OBJECT_NAME (videotestsrc));
+      g_object_set (pool, "name", name, NULL);
+      g_free (name);
+    }
   }
 
   config = gst_buffer_pool_get_config (pool);
@@ -980,6 +1002,11 @@ gst_video_test_src_decide_allocation (GstBaseSrc * bsrc, GstQuery * query)
   if (gst_query_find_allocation_meta (query, GST_VIDEO_META_API_TYPE, NULL)) {
     gst_buffer_pool_config_add_option (config,
         GST_BUFFER_POOL_OPTION_VIDEO_META);
+
+    if (gst_buffer_pool_has_option (pool,
+            GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT))
+      gst_buffer_pool_config_add_option (config,
+          GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT);
   }
   gst_buffer_pool_set_config (pool, config);
 
@@ -1068,7 +1095,9 @@ gst_video_test_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
   videotestsrc->running_time = 0;
   videotestsrc->n_frames = 0;
 
+  g_mutex_lock (&videotestsrc->cache_lock);
   gst_clear_buffer (&videotestsrc->cached);
+  g_mutex_unlock (&videotestsrc->cache_lock);
 
   GST_OBJECT_UNLOCK (videotestsrc);
 
@@ -1163,8 +1192,8 @@ gst_video_test_src_query (GstBaseSrc * bsrc, GstQuery * query)
             break;
         }
       }
-      /* fall through */
     }
+      /* FALLTHROUGH */
     default:
       res = GST_BASE_SRC_CLASS (parent_class)->query (bsrc, query);
       break;
@@ -1304,12 +1333,15 @@ gst_video_test_src_fill (GstPushSrc * psrc, GstBuffer * buffer)
   if (src->have_static_pattern) {
     GstVideoFrame sframe, dframe;
 
+    g_mutex_lock (&src->cache_lock);
     if (src->cached == NULL) {
       src->cached = gst_buffer_new_allocate (NULL, src->info.size, NULL);
 
       ret = fill_image (GST_PUSH_SRC (src), src->cached);
-      if (G_UNLIKELY (ret != GST_FLOW_OK))
+      if (G_UNLIKELY (ret != GST_FLOW_OK)) {
+        g_mutex_unlock (&src->cache_lock);
         goto fill_failed;
+      }
     } else {
       GST_LOG_OBJECT (src, "Reusing cached pattern buffer");
     }
@@ -1321,11 +1353,14 @@ gst_video_test_src_fill (GstPushSrc * psrc, GstBuffer * buffer)
     gst_video_frame_map (&sframe, &src->info, src->cached, GST_MAP_READ);
     gst_video_frame_map (&dframe, &src->info, buffer, GST_MAP_WRITE);
 
-    if (!gst_video_frame_copy (&dframe, &sframe))
+    if (!gst_video_frame_copy (&dframe, &sframe)) {
+      g_mutex_unlock (&src->cache_lock);
       goto copy_failed;
+    }
 
     gst_video_frame_unmap (&sframe);
     gst_video_frame_unmap (&dframe);
+    g_mutex_unlock (&src->cache_lock);
   } else {
     ret = fill_image (GST_PUSH_SRC (src), buffer);
     if (G_UNLIKELY (ret != GST_FLOW_OK))
@@ -1425,7 +1460,9 @@ gst_video_test_src_stop (GstBaseSrc * basesrc)
   src->n_lines = 0;
   src->lines = NULL;
 
+  g_mutex_lock (&src->cache_lock);
   gst_clear_buffer (&src->cached);
+  g_mutex_unlock (&src->cache_lock);
 
   return TRUE;
 }
@@ -1433,9 +1470,6 @@ gst_video_test_src_stop (GstBaseSrc * basesrc)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GST_DEBUG_CATEGORY_INIT (video_test_src_debug, "videotestsrc", 0,
-      "Video Test Source");
-
   return GST_ELEMENT_REGISTER (videotestsrc, plugin);
 }
 
