@@ -28,103 +28,99 @@
 # include <valgrind/valgrind.h>
 #endif
 
-#include <unistd.h>
-
 #include <gst/check/gstcheck.h>
 #include <gst/check/gstconsistencychecker.h>
 #include <gst/base/gstbasesrc.h>
+#include <gst/audio/audio.h>
 
 static GMainLoop *main_loop;
 
-/* make sure downstream gets a CAPS event before buffers are sent */
-GST_START_TEST (test_caps)
+/* fixtures */
+
+static void
+test_setup (void)
 {
-  GstElement *pipeline, *src, *adder, *sink;
-  GstStateChangeReturn state_res;
+  main_loop = g_main_loop_new (NULL, FALSE);
+}
+
+static void
+test_teardown (void)
+{
+  g_main_loop_unref (main_loop);
+  main_loop = NULL;
+}
+
+
+/* some test helpers */
+
+static GstElement *
+setup_pipeline (GstElement * adder, gint num_srcs)
+{
+  GstElement *pipeline, *src, *sink;
+  gint i;
+
+  pipeline = gst_pipeline_new ("pipeline");
+  if (!adder) {
+    adder = gst_element_factory_make ("adder", "adder");
+  }
+
+  sink = gst_element_factory_make ("fakesink", "sink");
+  gst_bin_add_many (GST_BIN (pipeline), adder, sink, NULL);
+  gst_element_link (adder, sink);
+
+  for (i = 0; i < num_srcs; i++) {
+    src = gst_element_factory_make ("audiotestsrc", NULL);
+    g_object_set (src, "wave", 4, NULL);        /* silence */
+    gst_bin_add (GST_BIN (pipeline), src);
+    gst_element_link (src, adder);
+  }
+  return pipeline;
+}
+
+static GstCaps *
+get_element_sink_pad_caps (GstElement * pipeline, const gchar * element_name)
+{
+  GstElement *sink;
   GstCaps *caps;
   GstPad *pad;
 
-  /* build pipeline */
-  pipeline = gst_pipeline_new ("pipeline");
-
-  src = gst_element_factory_make ("audiotestsrc", "src1");
-  g_object_set (src, "wave", 4, NULL);  /* silence */
-  adder = gst_element_factory_make ("adder", "adder");
-  sink = gst_element_factory_make ("fakesink", "sink");
-  gst_bin_add_many (GST_BIN (pipeline), src, adder, sink, NULL);
-
-  fail_unless (gst_element_link_many (src, adder, sink, NULL));
-
-  /* prepare playing */
-  state_res = gst_element_set_state (pipeline, GST_STATE_PAUSED);
-  fail_unless_equals_int (state_res, GST_STATE_CHANGE_ASYNC);
-
-  /* wait for preroll */
-  state_res = gst_element_get_state (pipeline, NULL, NULL, GST_CLOCK_TIME_NONE);
-  fail_unless_equals_int (state_res, GST_STATE_CHANGE_SUCCESS);
-
-  /* check caps on fakesink */
+  sink = gst_bin_get_by_name (GST_BIN (pipeline), "sink");
   pad = gst_element_get_static_pad (sink, "sink");
   caps = gst_pad_get_current_caps (pad);
-  fail_unless (caps != NULL);
-  gst_caps_unref (caps);
   gst_object_unref (pad);
+  gst_object_unref (sink);
 
-  gst_element_set_state (pipeline, GST_STATE_NULL);
-  gst_object_unref (pipeline);
+  return caps;
 }
 
-GST_END_TEST;
-
-/* check that caps set on the property are honoured */
-GST_START_TEST (test_filter_caps)
+static void
+set_state_and_wait (GstElement * pipeline, GstState state)
 {
-  GstElement *pipeline, *src, *adder, *sink;
   GstStateChangeReturn state_res;
-  GstCaps *filter_caps, *caps;
-  GstPad *pad;
 
-  filter_caps = gst_caps_new_simple ("audio/x-raw",
-      "format", G_TYPE_STRING, "F32LE",
-      "layout", G_TYPE_STRING, "interleaved",
-      "rate", G_TYPE_INT, 44100, "channels", G_TYPE_INT, 1, NULL);
-
-  /* build pipeline */
-  pipeline = gst_pipeline_new ("pipeline");
-
-  src = gst_element_factory_make ("audiotestsrc", NULL);
-  g_object_set (src, "wave", 4, NULL);  /* silence */
-  adder = gst_element_factory_make ("adder", NULL);
-  g_object_set (adder, "caps", filter_caps, NULL);
-  sink = gst_element_factory_make ("fakesink", "sink");
-  gst_bin_add_many (GST_BIN (pipeline), src, adder, sink, NULL);
-
-  fail_unless (gst_element_link_many (src, adder, sink, NULL));
-
-  /* prepare playing */
-  state_res = gst_element_set_state (pipeline, GST_STATE_PAUSED);
-  fail_unless_equals_int (state_res, GST_STATE_CHANGE_ASYNC);
+  /* prepare paused/playing */
+  state_res = gst_element_set_state (pipeline, state);
+  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
 
   /* wait for preroll */
   state_res = gst_element_get_state (pipeline, NULL, NULL, GST_CLOCK_TIME_NONE);
-  fail_unless_equals_int (state_res, GST_STATE_CHANGE_SUCCESS);
-
-  /* check caps on fakesink */
-  pad = gst_element_get_static_pad (sink, "sink");
-  caps = gst_pad_get_current_caps (pad);
-  fail_unless (caps != NULL);
-  GST_INFO_OBJECT (pipeline, "received caps: %" GST_PTR_FORMAT, caps);
-  fail_unless (gst_caps_is_equal_fixed (caps, filter_caps));
-  gst_caps_unref (caps);
-  gst_object_unref (pad);
-
-  gst_element_set_state (pipeline, GST_STATE_NULL);
-  gst_object_unref (pipeline);
-
-  gst_caps_unref (filter_caps);
+  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
 }
 
-GST_END_TEST;
+static void
+play_and_wait (GstElement * pipeline)
+{
+  GstStateChangeReturn state_res;
+
+  state_res = gst_element_set_state (pipeline, GST_STATE_PLAYING);
+  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+
+  GST_INFO ("running main loop");
+  g_main_loop_run (main_loop);
+
+  state_res = gst_element_set_state (pipeline, GST_STATE_NULL);
+  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+}
 
 static void
 message_received (GstBus * bus, GstMessage * message, GstPipeline * bin)
@@ -162,6 +158,73 @@ message_received (GstBus * bus, GstMessage * message, GstPipeline * bin)
   }
 }
 
+static GstBuffer *
+new_buffer (gsize num_bytes, GstClockTime ts, GstClockTime dur)
+{
+  GstBuffer *buffer = gst_buffer_new_and_alloc (num_bytes);
+
+  GST_BUFFER_TIMESTAMP (buffer) = ts;
+  GST_BUFFER_DURATION (buffer) = dur;
+  GST_DEBUG ("created buffer %p", buffer);
+  return buffer;
+}
+
+/* make sure downstream gets a CAPS event before buffers are sent */
+GST_START_TEST (test_caps)
+{
+  GstElement *pipeline;
+  GstCaps *caps;
+
+  /* build pipeline */
+  pipeline = setup_pipeline (NULL, 1);
+
+  /* prepare playing */
+  set_state_and_wait (pipeline, GST_STATE_PAUSED);
+
+  /* check caps on fakesink */
+  caps = get_element_sink_pad_caps (pipeline, "sink");
+  fail_unless (caps != NULL);
+  gst_caps_unref (caps);
+
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+  gst_object_unref (pipeline);
+}
+
+GST_END_TEST;
+
+/* check that caps set on the property are honoured */
+GST_START_TEST (test_filter_caps)
+{
+  GstElement *pipeline, *adder;
+  GstCaps *filter_caps, *caps;
+
+  filter_caps = gst_caps_new_simple ("audio/x-raw",
+      "format", G_TYPE_STRING, GST_AUDIO_NE (F32),
+      "layout", G_TYPE_STRING, "interleaved",
+      "rate", G_TYPE_INT, 44100, "channels", G_TYPE_INT, 1, NULL);
+
+  /* build pipeline */
+  adder = gst_element_factory_make ("adder", "adder");
+  g_object_set (adder, "caps", filter_caps, NULL);
+  pipeline = setup_pipeline (adder, 1);
+
+  /* prepare playing */
+  set_state_and_wait (pipeline, GST_STATE_PAUSED);
+
+  /* check caps on fakesink */
+  caps = get_element_sink_pad_caps (pipeline, "sink");
+  fail_unless (caps != NULL);
+  GST_INFO_OBJECT (pipeline, "received caps: %" GST_PTR_FORMAT, caps);
+  fail_unless (gst_caps_is_equal_fixed (caps, filter_caps));
+  gst_caps_unref (caps);
+
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+  gst_object_unref (pipeline);
+
+  gst_caps_unref (filter_caps);
+}
+
+GST_END_TEST;
 
 static GstFormat format = GST_FORMAT_UNDEFINED;
 static gint64 position = -1;
@@ -185,13 +248,11 @@ test_event_message_received (GstBus * bus, GstMessage * message,
   }
 }
 
-
 GST_START_TEST (test_event)
 {
   GstElement *bin, *src1, *src2, *adder, *sink;
   GstBus *bus;
   GstEvent *seek_event;
-  GstStateChangeReturn state_res;
   gboolean res;
   GstPad *srcpad, *sinkpad;
   GstStreamConsistency *chk_1, *chk_2, *chk_3;
@@ -245,7 +306,6 @@ GST_START_TEST (test_event)
   format = GST_FORMAT_UNDEFINED;
   position = -1;
 
-  main_loop = g_main_loop_new (NULL, FALSE);
   g_signal_connect (bus, "message::segment-done",
       (GCallback) test_event_message_received, bin);
   g_signal_connect (bus, "message::error", (GCallback) message_received, bin);
@@ -255,30 +315,17 @@ GST_START_TEST (test_event)
   GST_INFO ("starting test");
 
   /* prepare playing */
-  state_res = gst_element_set_state (bin, GST_STATE_PAUSED);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  /* wait for completion */
-  state_res = gst_element_get_state (bin, NULL, NULL, GST_CLOCK_TIME_NONE);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+  set_state_and_wait (bin, GST_STATE_PAUSED);
 
   res = gst_element_send_event (bin, seek_event);
   fail_unless (res == TRUE, NULL);
 
   /* run pipeline */
-  state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  GST_INFO ("running main loop");
-  g_main_loop_run (main_loop);
-
-  state_res = gst_element_set_state (bin, GST_STATE_NULL);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+  play_and_wait (bin);
 
   ck_assert_int_eq (position, 2 * GST_SECOND);
 
   /* cleanup */
-  g_main_loop_unref (main_loop);
   gst_consistency_checker_free (chk_1);
   gst_consistency_checker_free (chk_2);
   gst_consistency_checker_free (chk_3);
@@ -294,7 +341,7 @@ static GstEvent *play_seek_event = NULL;
 
 static void
 test_play_twice_message_received (GstBus * bus, GstMessage * message,
-    GstPipeline * bin)
+    GstElement * bin)
 {
   gboolean res;
   GstStateChangeReturn state_res;
@@ -306,25 +353,16 @@ test_play_twice_message_received (GstBus * bus, GstMessage * message,
     case GST_MESSAGE_SEGMENT_DONE:
       play_count++;
       if (play_count == 1) {
-        state_res = gst_element_set_state (GST_ELEMENT (bin), GST_STATE_READY);
+        state_res = gst_element_set_state (bin, GST_STATE_READY);
         ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
 
         /* prepare playing again */
-        state_res = gst_element_set_state (GST_ELEMENT (bin), GST_STATE_PAUSED);
-        ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+        set_state_and_wait (bin, GST_STATE_PAUSED);
 
-        /* wait for completion */
-        state_res =
-            gst_element_get_state (GST_ELEMENT (bin), NULL, NULL,
-            GST_CLOCK_TIME_NONE);
-        ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-        res = gst_element_send_event (GST_ELEMENT (bin),
-            gst_event_ref (play_seek_event));
+        res = gst_element_send_event (bin, gst_event_ref (play_seek_event));
         fail_unless (res == TRUE, NULL);
 
-        state_res =
-            gst_element_set_state (GST_ELEMENT (bin), GST_STATE_PLAYING);
+        state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
         ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
       } else {
         g_main_loop_quit (main_loop);
@@ -339,35 +377,20 @@ test_play_twice_message_received (GstBus * bus, GstMessage * message,
 
 GST_START_TEST (test_play_twice)
 {
-  GstElement *bin, *src1, *src2, *adder, *sink;
+  GstElement *bin, *adder;
   GstBus *bus;
   gboolean res;
-  GstStateChangeReturn state_res;
   GstPad *srcpad;
   GstStreamConsistency *consist;
 
   GST_INFO ("preparing test");
 
   /* build pipeline */
-  bin = gst_pipeline_new ("pipeline");
+  adder = gst_element_factory_make ("adder", "adder");
+  bin = setup_pipeline (adder, 2);
   bus = gst_element_get_bus (bin);
   gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
 
-  src1 = gst_element_factory_make ("audiotestsrc", "src1");
-  g_object_set (src1, "wave", 4, NULL); /* silence */
-  src2 = gst_element_factory_make ("audiotestsrc", "src2");
-  g_object_set (src2, "wave", 4, NULL); /* silence */
-  adder = gst_element_factory_make ("adder", "adder");
-  sink = gst_element_factory_make ("fakesink", "sink");
-  gst_bin_add_many (GST_BIN (bin), src1, src2, adder, sink, NULL);
-
-  res = gst_element_link (src1, adder);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (src2, adder);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (adder, sink);
-  fail_unless (res == TRUE, NULL);
-
   srcpad = gst_element_get_static_pad (adder, "src");
   consist = gst_consistency_checker_new (srcpad);
   gst_object_unref (srcpad);
@@ -379,7 +402,6 @@ GST_START_TEST (test_play_twice)
 
   play_count = 0;
 
-  main_loop = g_main_loop_new (NULL, FALSE);
   g_signal_connect (bus, "message::segment-done",
       (GCallback) test_play_twice_message_received, bin);
   g_signal_connect (bus, "message::error", (GCallback) message_received, bin);
@@ -389,14 +411,7 @@ GST_START_TEST (test_play_twice)
   GST_INFO ("starting test");
 
   /* prepare playing */
-  state_res = gst_element_set_state (bin, GST_STATE_PAUSED);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  /* wait for completion */
-  state_res =
-      gst_element_get_state (GST_ELEMENT (bin), NULL, NULL,
-      GST_CLOCK_TIME_NONE);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+  set_state_and_wait (bin, GST_STATE_PAUSED);
 
   res = gst_element_send_event (bin, gst_event_ref (play_seek_event));
   fail_unless (res == TRUE, NULL);
@@ -404,20 +419,13 @@ GST_START_TEST (test_play_twice)
   GST_INFO ("seeked");
 
   /* run pipeline */
-  state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  g_main_loop_run (main_loop);
-
-  state_res = gst_element_set_state (bin, GST_STATE_NULL);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+  play_and_wait (bin);
 
   ck_assert_int_eq (play_count, 2);
 
   /* cleanup */
-  g_main_loop_unref (main_loop);
   gst_consistency_checker_free (consist);
-  gst_event_ref (play_seek_event);
+  gst_event_unref (play_seek_event);
   gst_bus_remove_signal_watch (bus);
   gst_object_unref (bus);
   gst_object_unref (bin);
@@ -427,7 +435,7 @@ GST_END_TEST;
 
 GST_START_TEST (test_play_twice_then_add_and_play_again)
 {
-  GstElement *bin, *src1, *src2, *src3, *adder, *sink;
+  GstElement *bin, *src, *adder;
   GstBus *bus;
   gboolean res;
   GstStateChangeReturn state_res;
@@ -438,35 +446,20 @@ GST_START_TEST (test_play_twice_then_add_and_play_again)
   GST_INFO ("preparing test");
 
   /* build pipeline */
-  bin = gst_pipeline_new ("pipeline");
+  adder = gst_element_factory_make ("adder", "adder");
+  bin = setup_pipeline (adder, 2);
   bus = gst_element_get_bus (bin);
   gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
 
-  src1 = gst_element_factory_make ("audiotestsrc", "src1");
-  g_object_set (src1, "wave", 4, NULL); /* silence */
-  src2 = gst_element_factory_make ("audiotestsrc", "src2");
-  g_object_set (src2, "wave", 4, NULL); /* silence */
-  adder = gst_element_factory_make ("adder", "adder");
-  sink = gst_element_factory_make ("fakesink", "sink");
-  gst_bin_add_many (GST_BIN (bin), src1, src2, adder, sink, NULL);
-
   srcpad = gst_element_get_static_pad (adder, "src");
   consist = gst_consistency_checker_new (srcpad);
   gst_object_unref (srcpad);
 
-  res = gst_element_link (src1, adder);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (src2, adder);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (adder, sink);
-  fail_unless (res == TRUE, NULL);
-
   play_seek_event = gst_event_new_seek (1.0, GST_FORMAT_TIME,
       GST_SEEK_FLAG_SEGMENT | GST_SEEK_FLAG_FLUSH,
       GST_SEEK_TYPE_SET, (GstClockTime) 0,
       GST_SEEK_TYPE_SET, (GstClockTime) 2 * GST_SECOND);
 
-  main_loop = g_main_loop_new (NULL, FALSE);
   g_signal_connect (bus, "message::segment-done",
       (GCallback) test_play_twice_message_received, bin);
   g_signal_connect (bus, "message::error", (GCallback) message_received, bin);
@@ -480,14 +473,7 @@ GST_START_TEST (test_play_twice_then_add_and_play_again)
     GST_INFO ("starting test-loop %d", i);
 
     /* prepare playing */
-    state_res = gst_element_set_state (bin, GST_STATE_PAUSED);
-    ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-    /* wait for completion */
-    state_res =
-        gst_element_get_state (GST_ELEMENT (bin), NULL, NULL,
-        GST_CLOCK_TIME_NONE);
-    ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+    set_state_and_wait (bin, GST_STATE_PAUSED);
 
     res = gst_element_send_event (bin, gst_event_ref (play_seek_event));
     fail_unless (res == TRUE, NULL);
@@ -495,23 +481,17 @@ GST_START_TEST (test_play_twice_then_add_and_play_again)
     GST_INFO ("seeked");
 
     /* run pipeline */
-    state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-    ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-    g_main_loop_run (main_loop);
-
-    state_res = gst_element_set_state (bin, GST_STATE_READY);
-    ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+    play_and_wait (bin);
 
     ck_assert_int_eq (play_count, 2);
 
     /* plug another source */
     if (i == 0) {
-      src3 = gst_element_factory_make ("audiotestsrc", "src3");
-      g_object_set (src3, "wave", 4, NULL);     /* silence */
-      gst_bin_add (GST_BIN (bin), src3);
+      src = gst_element_factory_make ("audiotestsrc", NULL);
+      g_object_set (src, "wave", 4, NULL);      /* silence */
+      gst_bin_add (GST_BIN (bin), src);
 
-      res = gst_element_link (src3, adder);
+      res = gst_element_link (src, adder);
       fail_unless (res == TRUE, NULL);
     }
 
@@ -522,8 +502,7 @@ GST_START_TEST (test_play_twice_then_add_and_play_again)
   ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
 
   /* cleanup */
-  g_main_loop_unref (main_loop);
-  gst_event_ref (play_seek_event);
+  gst_event_unref (play_seek_event);
   gst_consistency_checker_free (consist);
   gst_bus_remove_signal_watch (bus);
   gst_object_unref (bus);
@@ -532,48 +511,6 @@ GST_START_TEST (test_play_twice_then_add_and_play_again)
 
 GST_END_TEST;
 
-
-static void
-test_live_seeking_eos_message_received (GstBus * bus, GstMessage * message,
-    GstPipeline * bin)
-{
-  GST_INFO ("bus message from \"%" GST_PTR_FORMAT "\": %" GST_PTR_FORMAT,
-      GST_MESSAGE_SRC (message), message);
-
-  switch (message->type) {
-    case GST_MESSAGE_EOS:
-      g_main_loop_quit (main_loop);
-      break;
-    default:
-      g_assert_not_reached ();
-      break;
-  }
-}
-
-static GstElement *
-test_live_seeking_try_audiosrc (const gchar * factory_name)
-{
-  GstElement *src;
-  GstStateChangeReturn state_res;
-
-  if (!(src = gst_element_factory_make (factory_name, NULL))) {
-    GST_INFO ("can't make '%s', skipping", factory_name);
-    return NULL;
-  }
-
-  /* Test that the audio source can get to ready, else skip */
-  state_res = gst_element_set_state (src, GST_STATE_READY);
-  gst_element_set_state (src, GST_STATE_NULL);
-
-  if (state_res == GST_STATE_CHANGE_FAILURE) {
-    GST_INFO_OBJECT (src, "can't go to ready, skipping");
-    gst_object_unref (src);
-    return NULL;
-  }
-
-  return src;
-}
-
 /* test failing seeks on live-sources */
 GST_START_TEST (test_live_seeking)
 {
@@ -582,16 +519,9 @@ GST_START_TEST (test_live_seeking)
   gboolean res;
   GstPad *srcpad;
   gint i;
-  GstStateChangeReturn state_res;
   GstStreamConsistency *consist;
-  /* don't use autoaudiosrc, as then we can't set anything here */
-  const gchar *audio_src_factories[] = {
-    "alsasrc",
-    "pulseaudiosrc"
-  };
 
   GST_INFO ("preparing test");
-  main_loop = NULL;
   play_seek_event = NULL;
 
   /* build pipeline */
@@ -599,19 +529,8 @@ GST_START_TEST (test_live_seeking)
   bus = gst_element_get_bus (bin);
   gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
 
-  for (i = 0; (i < G_N_ELEMENTS (audio_src_factories) && src1 == NULL); i++) {
-    src1 = test_live_seeking_try_audiosrc (audio_src_factories[i]);
-  }
-  if (!src1) {
-    /* normal audiosources behave differently than audiotestsrc */
-    src1 = gst_element_factory_make ("audiotestsrc", "src1");
-    g_object_set (src1, "wave", 4, "is-live", TRUE, NULL);      /* silence */
-  } else {
-    /* live sources ignore seeks, force eos after 2 sec (4 buffers half second
-     * each)
-     */
-    g_object_set (src1, "num-buffers", 4, "blocksize", 44100, NULL);
-  }
+  src1 = gst_element_factory_make ("audiotestsrc", "src1");
+  g_object_set (src1, "wave", 4, "is-live", TRUE, NULL);        /* silence */
 
   ac1 = gst_element_factory_make ("audioconvert", "ac1");
   src2 = gst_element_factory_make ("audiotestsrc", "src2");
@@ -621,13 +540,9 @@ GST_START_TEST (test_live_seeking)
   sink = gst_element_factory_make ("fakesink", "sink");
   gst_bin_add_many (GST_BIN (bin), src1, ac1, src2, ac2, adder, sink, NULL);
 
-  res = gst_element_link (src1, ac1);
+  res = gst_element_link_many (src1, ac1, adder, NULL);
   fail_unless (res == TRUE, NULL);
-  res = gst_element_link (ac1, adder);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (src2, ac2);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (ac2, adder);
+  res = gst_element_link_many (src2, ac2, adder, NULL);
   fail_unless (res == TRUE, NULL);
   res = gst_element_link (adder, sink);
   fail_unless (res == TRUE, NULL);
@@ -637,11 +552,9 @@ GST_START_TEST (test_live_seeking)
       GST_SEEK_TYPE_SET, (GstClockTime) 0,
       GST_SEEK_TYPE_SET, (GstClockTime) 2 * GST_SECOND);
 
-  main_loop = g_main_loop_new (NULL, FALSE);
   g_signal_connect (bus, "message::error", (GCallback) message_received, bin);
   g_signal_connect (bus, "message::warning", (GCallback) message_received, bin);
-  g_signal_connect (bus, "message::eos",
-      (GCallback) test_live_seeking_eos_message_received, bin);
+  g_signal_connect (bus, "message::eos", (GCallback) message_received, bin);
 
   srcpad = gst_element_get_static_pad (adder, "src");
   consist = gst_consistency_checker_new (srcpad);
@@ -655,14 +568,7 @@ GST_START_TEST (test_live_seeking)
     GST_INFO ("starting test-loop %d", i);
 
     /* prepare playing */
-    state_res = gst_element_set_state (bin, GST_STATE_PAUSED);
-    ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-    /* wait for completion */
-    state_res =
-        gst_element_get_state (GST_ELEMENT (bin), NULL, NULL,
-        GST_CLOCK_TIME_NONE);
-    ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+    set_state_and_wait (bin, GST_STATE_PAUSED);
 
     res = gst_element_send_event (bin, gst_event_ref (play_seek_event));
     fail_unless (res == TRUE, NULL);
@@ -670,15 +576,7 @@ GST_START_TEST (test_live_seeking)
     GST_INFO ("seeked");
 
     /* run pipeline */
-    state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-    ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-    GST_INFO ("playing");
-
-    g_main_loop_run (main_loop);
-
-    state_res = gst_element_set_state (bin, GST_STATE_NULL);
-    ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+    play_and_wait (bin);
 
     gst_consistency_checker_reset (consist);
   }
@@ -686,10 +584,9 @@ GST_START_TEST (test_live_seeking)
   /* cleanup */
   GST_INFO ("cleaning up");
   gst_consistency_checker_free (consist);
-  if (main_loop)
-    g_main_loop_unref (main_loop);
   if (play_seek_event)
     gst_event_unref (play_seek_event);
+  gst_bus_remove_signal_watch (bus);
   gst_object_unref (bus);
   gst_object_unref (bin);
 }
@@ -713,12 +610,10 @@ GST_START_TEST (test_add_pad)
   gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
 
   src1 = gst_element_factory_make ("audiotestsrc", "src1");
-  g_object_set (src1, "num-buffers", 4, NULL);
-  g_object_set (src1, "wave", 4, NULL); /* silence */
+  g_object_set (src1, "num-buffers", 4, "wave", /* silence */ 4, NULL);
   src2 = gst_element_factory_make ("audiotestsrc", "src2");
   /* one buffer less, we connect with 1 buffer of delay */
-  g_object_set (src2, "num-buffers", 3, NULL);
-  g_object_set (src2, "wave", 4, NULL); /* silence */
+  g_object_set (src2, "num-buffers", 3, "wave", /* silence */ 4, NULL);
   adder = gst_element_factory_make ("adder", "adder");
   sink = gst_element_factory_make ("fakesink", "sink");
   gst_bin_add_many (GST_BIN (bin), src1, adder, sink, NULL);
@@ -731,7 +626,6 @@ GST_START_TEST (test_add_pad)
   srcpad = gst_element_get_static_pad (adder, "src");
   gst_object_unref (srcpad);
 
-  main_loop = g_main_loop_new (NULL, FALSE);
   g_signal_connect (bus, "message::segment-done", (GCallback) message_received,
       bin);
   g_signal_connect (bus, "message::error", (GCallback) message_received, bin);
@@ -741,14 +635,7 @@ GST_START_TEST (test_add_pad)
   GST_INFO ("starting test");
 
   /* prepare playing */
-  state_res = gst_element_set_state (bin, GST_STATE_PAUSED);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  /* wait for completion */
-  state_res =
-      gst_element_get_state (GST_ELEMENT (bin), NULL, NULL,
-      GST_CLOCK_TIME_NONE);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+  set_state_and_wait (bin, GST_STATE_PAUSED);
 
   /* add other element */
   gst_bin_add_many (GST_BIN (bin), src2, NULL);
@@ -762,16 +649,9 @@ GST_START_TEST (test_add_pad)
   ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
 
   /* now play all */
-  state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  g_main_loop_run (main_loop);
-
-  state_res = gst_element_set_state (bin, GST_STATE_NULL);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+  play_and_wait (bin);
 
   /* cleanup */
-  g_main_loop_unref (main_loop);
   gst_bus_remove_signal_watch (bus);
   gst_object_unref (bus);
   gst_object_unref (bin);
@@ -796,8 +676,7 @@ GST_START_TEST (test_remove_pad)
   gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
 
   src = gst_element_factory_make ("audiotestsrc", "src");
-  g_object_set (src, "num-buffers", 4, NULL);
-  g_object_set (src, "wave", 4, NULL);
+  g_object_set (src, "num-buffers", 4, "wave", 4, NULL);
   adder = gst_element_factory_make ("adder", "adder");
   sink = gst_element_factory_make ("fakesink", "sink");
   gst_bin_add_many (GST_BIN (bin), src, adder, sink, NULL);
@@ -808,13 +687,12 @@ GST_START_TEST (test_remove_pad)
   fail_unless (res == TRUE, NULL);
 
   /* create an unconnected sinkpad in adder */
-  pad = gst_element_get_request_pad (adder, "sink_%u");
+  pad = gst_element_request_pad_simple (adder, "sink_%u");
   fail_if (pad == NULL, NULL);
 
   srcpad = gst_element_get_static_pad (adder, "src");
   gst_object_unref (srcpad);
 
-  main_loop = g_main_loop_new (NULL, FALSE);
   g_signal_connect (bus, "message::segment-done", (GCallback) message_received,
       bin);
   g_signal_connect (bus, "message::error", (GCallback) message_received, bin);
@@ -844,16 +722,9 @@ GST_START_TEST (test_remove_pad)
   ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
 
   /* now play all */
-  state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  g_main_loop_run (main_loop);
-
-  state_res = gst_element_set_state (bin, GST_STATE_NULL);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+  play_and_wait (bin);
 
   /* cleanup */
-  g_main_loop_unref (main_loop);
   gst_bus_remove_signal_watch (bus);
   gst_object_unref (G_OBJECT (bus));
   gst_object_unref (G_OBJECT (bin));
@@ -912,17 +783,13 @@ GST_START_TEST (test_clip)
 
   /* create an unconnected sinkpad in adder, should also automatically activate
    * the pad */
-  sinkpad = gst_element_get_request_pad (adder, "sink_%u");
+  sinkpad = gst_element_request_pad_simple (adder, "sink_%u");
   fail_if (sinkpad == NULL, NULL);
 
   gst_pad_send_event (sinkpad, gst_event_new_stream_start ("test"));
 
   caps = gst_caps_new_simple ("audio/x-raw",
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-      "format", G_TYPE_STRING, "S16BE",
-#else
-      "format", G_TYPE_STRING, "S16LE",
-#endif
+      "format", G_TYPE_STRING, GST_AUDIO_NE (S16),
       "layout", G_TYPE_STRING, "interleaved",
       "rate", G_TYPE_INT, 44100, "channels", G_TYPE_INT, 2, NULL);
 
@@ -938,39 +805,27 @@ GST_START_TEST (test_clip)
   gst_pad_send_event (sinkpad, event);
 
   /* should be clipped and ok */
-  buffer = gst_buffer_new_and_alloc (44100);
-  GST_BUFFER_TIMESTAMP (buffer) = 0;
-  GST_BUFFER_DURATION (buffer) = 250 * GST_MSECOND;
-  GST_DEBUG ("pushing buffer %p", buffer);
+  buffer = new_buffer (44100, 0, 250 * GST_MSECOND);
   ret = gst_pad_chain (sinkpad, buffer);
   ck_assert_int_eq (ret, GST_FLOW_OK);
   fail_unless (handoff_buffer == NULL);
 
   /* should be partially clipped */
-  buffer = gst_buffer_new_and_alloc (44100);
-  GST_BUFFER_TIMESTAMP (buffer) = 900 * GST_MSECOND;
-  GST_BUFFER_DURATION (buffer) = 250 * GST_MSECOND;
-  GST_DEBUG ("pushing buffer %p", buffer);
+  buffer = new_buffer (44100, 900 * GST_MSECOND, 250 * GST_MSECOND);
   ret = gst_pad_chain (sinkpad, buffer);
   ck_assert_int_eq (ret, GST_FLOW_OK);
   fail_unless (handoff_buffer != NULL);
   gst_buffer_replace (&handoff_buffer, NULL);
 
   /* should not be clipped */
-  buffer = gst_buffer_new_and_alloc (44100);
-  GST_BUFFER_TIMESTAMP (buffer) = 1 * GST_SECOND;
-  GST_BUFFER_DURATION (buffer) = 250 * GST_MSECOND;
-  GST_DEBUG ("pushing buffer %p", buffer);
+  buffer = new_buffer (44100, 1 * GST_SECOND, 250 * GST_MSECOND);
   ret = gst_pad_chain (sinkpad, buffer);
   ck_assert_int_eq (ret, GST_FLOW_OK);
   fail_unless (handoff_buffer != NULL);
   gst_buffer_replace (&handoff_buffer, NULL);
 
   /* should be clipped and ok */
-  buffer = gst_buffer_new_and_alloc (44100);
-  GST_BUFFER_TIMESTAMP (buffer) = 2 * GST_SECOND;
-  GST_BUFFER_DURATION (buffer) = 250 * GST_MSECOND;
-  GST_DEBUG ("pushing buffer %p", buffer);
+  buffer = new_buffer (44100, 2 * GST_SECOND, 250 * GST_MSECOND);
   ret = gst_pad_chain (sinkpad, buffer);
   ck_assert_int_eq (ret, GST_FLOW_OK);
   fail_unless (handoff_buffer == NULL);
@@ -1021,14 +876,7 @@ GST_START_TEST (test_duration_is_max)
   GST_BASE_SRC (src[2])->segment.duration = 2000;
 
   /* set to playing */
-  state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-  fail_unless (state_res != GST_STATE_CHANGE_FAILURE, NULL);
-
-  /* wait for completion */
-  state_res =
-      gst_element_get_state (GST_ELEMENT (bin), NULL, NULL,
-      GST_CLOCK_TIME_NONE);
-  fail_unless (state_res != GST_STATE_CHANGE_FAILURE, NULL);
+  set_state_and_wait (bin, GST_STATE_PLAYING);
 
   res = gst_element_query_duration (GST_ELEMENT (bin), format, &duration);
   fail_unless (res, NULL);
@@ -1077,14 +925,7 @@ GST_START_TEST (test_duration_unknown_overrides)
   GST_BASE_SRC (src[2])->segment.duration = 2000;
 
   /* set to playing */
-  state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-  fail_unless (state_res != GST_STATE_CHANGE_FAILURE, NULL);
-
-  /* wait for completion */
-  state_res =
-      gst_element_get_state (GST_ELEMENT (bin), NULL, NULL,
-      GST_CLOCK_TIME_NONE);
-  fail_unless (state_res != GST_STATE_CHANGE_FAILURE, NULL);
+  set_state_and_wait (bin, GST_STATE_PLAYING);
 
   res = gst_element_query_duration (GST_ELEMENT (bin), format, &duration);
   fail_unless (res, NULL);
@@ -1125,40 +966,23 @@ loop_segment_done (GstBus * bus, GstMessage * message, GstElement * bin)
 
 GST_START_TEST (test_loop)
 {
-  GstElement *bin, *src1, *src2, *adder, *sink;
+  GstElement *bin;
   GstBus *bus;
   GstEvent *seek_event;
-  GstStateChangeReturn state_res;
   gboolean res;
 
   GST_INFO ("preparing test");
 
   /* build pipeline */
-  bin = gst_pipeline_new ("pipeline");
+  bin = setup_pipeline (NULL, 2);
   bus = gst_element_get_bus (bin);
   gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
 
-  src1 = gst_element_factory_make ("audiotestsrc", "src1");
-  g_object_set (src1, "wave", 4, NULL); /* silence */
-  src2 = gst_element_factory_make ("audiotestsrc", "src2");
-  g_object_set (src2, "wave", 4, NULL); /* silence */
-  adder = gst_element_factory_make ("adder", "adder");
-  sink = gst_element_factory_make ("fakesink", "sink");
-  gst_bin_add_many (GST_BIN (bin), src1, src2, adder, sink, NULL);
-
-  res = gst_element_link (src1, adder);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (src2, adder);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (adder, sink);
-  fail_unless (res == TRUE, NULL);
-
   seek_event = gst_event_new_seek (1.0, GST_FORMAT_TIME,
       GST_SEEK_FLAG_SEGMENT | GST_SEEK_FLAG_FLUSH,
       GST_SEEK_TYPE_SET, (GstClockTime) 0,
       GST_SEEK_TYPE_SET, (GstClockTime) 1 * GST_SECOND);
 
-  main_loop = g_main_loop_new (NULL, FALSE);
   g_signal_connect (bus, "message::segment-done",
       (GCallback) loop_segment_done, bin);
   g_signal_connect (bus, "message::error", (GCallback) message_received, bin);
@@ -1168,27 +992,17 @@ GST_START_TEST (test_loop)
   GST_INFO ("starting test");
 
   /* prepare playing */
-  state_res = gst_element_set_state (bin, GST_STATE_PAUSED);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  /* wait for completion */
-  state_res = gst_element_get_state (bin, NULL, NULL, GST_CLOCK_TIME_NONE);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+  set_state_and_wait (bin, GST_STATE_PAUSED);
 
   res = gst_element_send_event (bin, seek_event);
   fail_unless (res == TRUE, NULL);
 
   /* run pipeline */
-  state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
+  play_and_wait (bin);
 
-  GST_INFO ("running main loop");
-  g_main_loop_run (main_loop);
-
-  state_res = gst_element_set_state (bin, GST_STATE_NULL);
+  fail_unless (looped);
 
   /* cleanup */
-  g_main_loop_unref (main_loop);
   gst_bus_remove_signal_watch (bus);
   gst_object_unref (bus);
   gst_object_unref (bin);
@@ -1196,6 +1010,7 @@ GST_START_TEST (test_loop)
 
 GST_END_TEST;
 
+#if 0
 GST_START_TEST (test_flush_start_flush_stop)
 {
   GstPadTemplate *sink_template;
@@ -1230,9 +1045,8 @@ GST_START_TEST (test_flush_start_flush_stop)
 
   gst_element_link (adder, sink);
 
-  gst_element_set_state (pipeline, GST_STATE_PLAYING);
-  fail_unless (gst_element_get_state (pipeline, NULL, NULL,
-          GST_CLOCK_TIME_NONE) == GST_STATE_CHANGE_SUCCESS);
+  /* prepare playing */
+  set_state_and_wait (bin, GST_STATE_PLAYING);
 
   adder_src = gst_element_get_static_pad (adder, "src");
   fail_if (GST_PAD_IS_FLUSHING (adder_src));
@@ -1253,7 +1067,7 @@ GST_START_TEST (test_flush_start_flush_stop)
 }
 
 GST_END_TEST;
-
+#endif
 
 static Suite *
 adder_suite (void)
@@ -1274,7 +1088,13 @@ adder_suite (void)
   tcase_add_test (tc_chain, test_duration_is_max);
   tcase_add_test (tc_chain, test_duration_unknown_overrides);
   tcase_add_test (tc_chain, test_loop);
+  /* This test is racy and occasionally fails in interesting ways
+   * https://bugzilla.gnome.org/show_bug.cgi?id=708891
+   * It's unlikely that it will ever be fixed for adder, works with audiomixer */
+#if 0
   tcase_add_test (tc_chain, test_flush_start_flush_stop);
+#endif
+  tcase_add_checked_fixture (tc_chain, test_setup, test_teardown);
 
   /* Use a longer timeout */
 #ifdef HAVE_VALGRIND
