@@ -74,10 +74,10 @@ gst_test_scope_class_init (GstTestScopeClass * g_class)
       "Visualization",
       "Dummy test scope", "Stefan Kost <ensonic@users.sf.net>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_test_scope_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_test_scope_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_test_scope_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_test_scope_sink_template);
 }
 
 static void
